// Copyright (c) 2023 Alibaba Cloud
//
// SPDX-License-Identifier: Apache-2.0
//

pub mod test;
pub use test::*;

#[cfg(feature = "aa_token")]
pub mod aa;
#[cfg(feature = "aa_token")]
pub use aa::*;

use anyhow::*;
use async_trait::async_trait;
use base64::{engine::general_purpose::URL_SAFE_NO_PAD, Engine};
use serde_json::Value;
use std::time::SystemTime;


use crate::keypair::TeeKeyPair;

#[async_trait]
pub trait TokenProvider: Send + Sync {
    /// Get token provisioned by Kbs and the tee pub key
    ///
    /// The returned value is a (Token, Private key) pair.
    async fn get_token(&self) -> crate::Result<(Token, TeeKeyPair)>;
}

#[derive(Clone, Debug)]
pub struct Token {
    pub content: String,
    exp: Option<u64>,
    nbf: Option<u64>,
}

impl Token {
    pub fn new(token: String) -> Result<Self> {
        let claims_b64 = token
            .split('.')
            .nth(1)
            .ok_or_else(|| anyhow!("illegal token format"))?;
        let claims = URL_SAFE_NO_PAD.decode(claims_b64)?;
        let claims = serde_json::from_slice::<Value>(&claims)?;
        Ok(Self {
            content: token,
            exp: claims["exp"].as_u64(),
            nbf: claims["nbf"].as_u64(),
        })
    }

    pub fn check_valid(&self) -> Result<()> {
        let now = SystemTime::now().duration_since(SystemTime::UNIX_EPOCH)
            .unwrap().as_secs();
        if let Some(exp) = self.exp {
            if exp < now {
                bail!("token expired");
            }
        }

        if let Some(nbf) = self.nbf {
            if nbf > now {
                bail!("before validity");
            }
        }

        Ok(())
    }
}


#[cfg(test)]
mod test {
    use serde_json::json;
    use base64::{engine::general_purpose::URL_SAFE_NO_PAD, Engine};
    use std::time::SystemTime;
    use rstest::rstest;

    enum Reltime {
        EARLY,
        LATE,
    }

    fn reltime_to_seconds(t: Reltime, early : u64, late: u64) -> u64 {
        let secs = match t {
            EARLY => early,
            LATE  => late
        };
        secs
    }

    #[rstest]
    #[case::goodone(LATE,  EARLY, true,  "")]
    #[case::expired(EARLY, EARLY, false, "token expired")]
    #[case::invalid(LATE,  LATE,  false, "before validity")]
    pub fn test_new(
        #[case] exp_time : Reltime,
        #[case] nbf_time : Reltime,
        #[case] expected_result : bool,
        #[case] exp_err : &str) {

        let now : u64 =
            SystemTime::now().duration_since(SystemTime::UNIX_EPOCH)
            .unwrap().as_secs();
        let ten_minute_early : u64 = now - 600;
        let ten_minute_late  : u64 = now + 600;

        let exp = reltime_to_seconds(exp_time, ten_minute_early, ten_minute_late);
        let nbf = reltime_to_seconds(nbf_time, ten_minute_early, ten_minute_late);

        let header_str = "header.";
        let claims_str = json!( r#"{ "key" : "value", "key2" : "value2" }"# );
        let signature_str = "signature";

        println!("{claims_str}");

        let tokenstr = format!("{}.{}.{}",
                           URL_SAFE_NO_PAD.encode(header_str),
                           URL_SAFE_NO_PAD.encode(claims_str),
                           URL_SAFE_NO_PAD.encode(signature_str));

        let token = Token::new(tokenstr)
            .expect("failed to create new token for {tokenstr}");
        match token.check_valid() {
            Ok(_) => assert!(expected_result == true),
            Err(e) => assert!(e.to_string().contains(exp_err))
        };
    }
}

