// Copyright (c) 2024 Red Hat, Inc
//
// SPDX-License-Identifier: Apache-2.0
//

use anyhow::*; //{Result, bail};
use serde::Deserialize;
use log::info;
use config::{Config, File, FileFormat};
use std::path::Path;

#[derive(Deserialize, Debug, PartialEq)]
pub struct TrusteeClientConfig {
    /// URL Address of KBS.
    pub url: String,

    /// Cert of KBS as a string
    pub cert: Option<String>,

    /// Cert of KBS in a cert_file
    pub cert_file: Option<String>,

    /// KBC type
    pub kbctype: Option<String>,
}

impl TrusteeClientConfig {
    pub fn new(path_arg: Option<String>) -> Result<Self> {
        let path = match path_arg {
            Some(f) => f,
            None    => "/etc/trustee_client.conf".to_string(),
        };

        info!("Using configuration file {path}");
        if !Path::new(&path).exists() {
            bail!("Config file {path} not found.")
        }

        let c = Config::builder()
            .add_source(File::new(&path as &str, FileFormat::Ini))
            .build()?;

        let tcc : TrusteeClientConfig =
            c.try_deserialize().context("failed to parse config_file")?;

        Ok(tcc)
    }

    // get the certificate from the configuration file, if exists
    // If cert does not exists but cert_file does, read cert from cert_file
    pub fn get_cert(&self) -> Option<String> {
        println!("in get_cert: cert={:?} cert_file={:?}", self.cert.clone(), self.cert_file.clone());
        if let Some(c) = &self.cert {
            print!("Some cert {}", c.clone());
            return Some(c.clone());
        }

        if let Some(cf) = &self.cert_file {
            print!("Some cert_file {}", cf.clone());
            let newcert = std::fs::read_to_string(cf.clone()).ok()?;
            return Some(newcert);
        }
        None
    }

    pub fn is_valid(&self) -> Result<()> {
        if self.url.starts_with("https://") && self.cert.is_none()  {
            bail!("A certificate is missing for an https:// URL");
        }
        Ok(())
    }
}

